@@ -10,6 +10,21 @@ from evelink.parsing.orders import parse_market_orders
 from evelink.parsing.wallet_journal import parse_wallet_journal
 from evelink.parsing.wallet_transactions import parse_wallet_transactions
 
+
+class auto_call(api.auto_call):
+    """Extends 'evelink.api.auto_call' to add 'Char.char_id' as an api 
+    request argument.
+    """
+
+    def __init__(self, path, map_params=None, **kw):
+        map_params = map_params if map_params else {}
+        map_params['char_id'] = 'characterID'
+
+        super(auto_call, self).__init__(
+            path, prop_to_param=('char_id',), map_params=map_params, **kw
+        )
+
+
 class Char(object):
     """Wrapper around /char/ of the EVE API.
 
@@ -20,7 +35,8 @@ class Char(object):
         self.api = api
         self.char_id = char_id
 
-    def assets(self):
+    @auto_call('char/AssetList')
+    def assets(self, api_result=None):
         """Get information about corp assets.
 
         Each item is a dict, with keys 'id', 'item_type_id',
@@ -42,44 +58,32 @@ class Char(object):
         the top-level values as "containers" with no fields except for
         "contents" and "location_id".
         """
-        api_result = self.api.get('char/AssetList', {'characterID': self.char_id})
 
         return api.APIResult(parse_assets(api_result.result), api_result.timestamp, api_result.expires)
 
-    def contract_bids(self):
+    @auto_call('char/ContractBids')
+    def contract_bids(self, api_result=None):
         """Lists the latest bids that have been made to any recent auctions."""
-        api_result = self.api.get('char/ContractBids', {'characterID': self.char_id})
-
         return api.APIResult(parse_contract_bids(api_result.result), api_result.timestamp, api_result.expires)
 
-    def contract_items(self, contract_id):
+    @auto_call('char/ContractItems', map_params={'contract_id': 'contractID'})
+    def contract_items(self, contract_id, api_result=None):
         """Lists items that a specified contract contains"""
-        api_result = self.api.get('char/ContractItems',
-            {'characterID': self.char_id, 'contractID': contract_id})
-
         return api.APIResult(parse_contract_items(api_result.result), api_result.timestamp, api_result.expires)
 
-    def contracts(self):
+    @auto_call('char/Contracts')
+    def contracts(self, api_result=None):
         """Returns a record of all contracts for a specified character"""
-        api_result = self.api.get('char/Contracts', {'characterID': self.char_id})
         return api.APIResult(parse_contracts(api_result.result), api_result.timestamp, api_result.expires)
 
-    def wallet_journal(self, before_id=None, limit=None):
+    @auto_call('char/WalletJournal', map_params={'before_id': 'fromID', 'limit': 'rowCount'})
+    def wallet_journal(self, before_id=None, limit=None, api_result=None):
         """Returns a complete record of all wallet activity for a specified character"""
-        params = {'characterID': self.char_id}
-        if before_id is not None:
-            params['fromID'] = before_id
-        if limit is not None:
-            params['rowCount'] = limit
-        api_result = self.api.get('char/WalletJournal', params)
-
         return api.APIResult(parse_wallet_journal(api_result.result), api_result.timestamp, api_result.expires)
 
-    def wallet_info(self):
+    @auto_call('char/AccountBalance')
+    def wallet_info(self, api_result=None):
         """Return a given character's wallet."""
-        api_result = self.api.get('char/AccountBalance',
-            {'characterID': self.char_id})
-
         rowset = api_result.result.find('rowset')
         row = rowset.find('row')
         result = {
@@ -94,42 +98,29 @@ class Char(object):
         api_result = self.wallet_info()
         return api.APIResult(api_result.result['balance'], api_result.timestamp, api_result.expires)
 
-    def wallet_transactions(self, before_id=None, limit=None):
+    @auto_call('char/WalletTransactions', map_params={'before_id': 'fromID', 'limit': 'rowCount'})
+    def wallet_transactions(self, before_id=None, limit=None, api_result=None):
         """Returns wallet transactions for a character."""
-        params = {'characterID': self.char_id}
-        if before_id is not None:
-            params['fromID'] = before_id
-        if limit is not None:
-            params['rowCount'] = limit
-        api_result = self.api.get('char/WalletTransactions', params)
-
         return api.APIResult(parse_wallet_transactions(api_result.result), api_result.timestamp, api_result.expires)
 
-    def industry_jobs(self):
+    @auto_call('char/IndustryJobs')
+    def industry_jobs(self, api_result=None):
         """Get a list of jobs for a character"""
-        api_result = self.api.get('char/IndustryJobs', {'characterID': self.char_id})
-
         return api.APIResult(parse_industry_jobs(api_result.result), api_result.timestamp, api_result.expires)
 
-    def kills(self, before_kill=None):
+    @auto_call('char/KillLog', map_params={'before_kill': 'beforeKillID'})
+    def kills(self, before_kill=None, api_result=None):
         """Look up recent kills for a character.
 
         before_kill:
             Optional. Only show kills before this kill id. (Used for paging.)
         """
 
-        params = {'characterID': self.char_id}
-        if before_kill is not None:
-            params['beforeKillID'] = before_kill
-        api_result = self.api.get('char/KillLog', params)
-
         return api.APIResult(parse_kills(api_result.result), api_result.timestamp, api_result.expires)
 
-    def notifications(self):
+    @auto_call('char/Notifications')
+    def notifications(self, api_result=None):
         """Returns the message headers for notifications."""
-        api_result = self.api.get('char/Notifications',
-            {'characterID': self.char_id})
-
         result = {}
         rowset = api_result.result.find('rowset')
         for row in rowset.findall('row'):
@@ -145,11 +136,9 @@ class Char(object):
 
         return api.APIResult(result, api_result.timestamp, api_result.expires)
 
-    def notification_texts(self, notification_ids):
+    @auto_call('char/NotificationTexts', map_params={'notification_ids': 'IDs'})
+    def notification_texts(self, notification_ids, api_result=None):
         """Returns the message bodies for notifications."""
-        api_result = self.api.get('char/NotificationTexts',
-            {'characterID': self.char_id, 'IDs': notification_ids})
-
         result = {}
         rowset = api_result.result.find('rowset')
         for row in rowset.findall('row'):
@@ -165,11 +154,9 @@ class Char(object):
 
         return api.APIResult(result, api_result.timestamp, api_result.expires)
 
-    def standings(self):
+    @auto_call('char/Standings')
+    def standings(self, api_result=None):
         """Returns the standings towards a character from NPC entities."""
-        api_result = self.api.get('char/Standings',
-            {'characterID': self.char_id})
-
         result = {}
         rowsets = {}
         for rowset in api_result.result.find('characterNPCStandings').findall('rowset'):
@@ -181,7 +168,7 @@ class Char(object):
             'factions': 'factions',
         }
 
-        for key, rowset_name in _name_map.iteritems():
+        for key, rowset_name in _name_map.items():
             result[key] = {}
             for row in rowsets[rowset_name].findall('row'):
                 a = row.attrib
@@ -194,11 +181,9 @@ class Char(object):
 
         return api.APIResult(result, api_result.timestamp, api_result.expires)
 
-    def character_sheet(self):
+    @auto_call('char/CharacterSheet')
+    def character_sheet(self, api_result=None):
         """Returns attributes relating to a specific character."""
-        api_result = self.api.get('char/CharacterSheet',
-            {'characterID': self.char_id})
-
         _str, _int, _float, _bool, _ts = api.elem_getters(api_result.result)
         result = {
             'id': _int('characterID'),
@@ -261,7 +246,7 @@ class Char(object):
             result['certificates'].add(int(cert.attrib['certificateID']))
 
         result['roles'] = {}
-        for our_role, ccp_role in constants.Char().corp_roles.iteritems():
+        for our_role, ccp_role in constants.Char().corp_roles.items():
             result['roles'][our_role] = {}
             for role in rowsets[ccp_role]:
                 a = role.attrib
@@ -282,26 +267,19 @@ class Char(object):
 
         return api.APIResult(result, api_result.timestamp, api_result.expires)
 
-    def contacts(self):
+    @auto_call('char/ContactList')
+    def contacts(self, api_result=None):
         """Return a character's personal, corp and alliance contact lists."""
-        api_result = self.api.get('char/ContactList',
-            {'characterID': self.char_id})
-
         return api.APIResult(parse_contact_list(api_result.result), api_result.timestamp, api_result.expires)
 
-    def orders(self):
+    @auto_call('char/MarketOrders')
+    def orders(self, api_result=None):
         """Return a given character's buy and sell orders."""
-        api_result = self.api.get('char/MarketOrders',
-            {'characterID': self.char_id})
-
         return api.APIResult(parse_market_orders(api_result.result), api_result.timestamp, api_result.expires)
 
-    def research(self):
+    @auto_call('char/Research')
+    def research(self, api_result=None):
         """Returns information about the agents with whom the character is doing research."""
-
-        api_result = self.api.get('char/Research',
-            {'characterID': self.char_id})
-
         rowset = api_result.result.find('rowset')
         rows = rowset.findall('row')
         result = {}
@@ -318,11 +296,9 @@ class Char(object):
 
         return api.APIResult(result, api_result.timestamp, api_result.expires)
 
-    def current_training(self):
+    @auto_call('char/SkillInTraining')
+    def current_training(self, api_result=None):
         """Returns the skill that is currently being trained by a specified character"""
-
-        api_result = self.api.get('char/SkillInTraining', {'characterID': self.char_id})
-
         _str, _int, _float, _bool, _ts = api.elem_getters(api_result.result)
         result = {
             'start_ts': _ts('trainingStartTime'),
@@ -337,11 +313,9 @@ class Char(object):
 
         return api.APIResult(result, api_result.timestamp, api_result.expires)
 
-    def skill_queue(self):
+    @auto_call('char/SkillQueue')
+    def skill_queue(self, api_result=None):
         """returns the skill queue of the character"""
-        api_result = self.api.get('char/SkillQueue',
-            {'characterID': self.char_id})
-
         rowset = api_result.result.find('rowset')
         rows = rowset.findall('row')
         result = []
@@ -361,11 +335,9 @@ class Char(object):
 
         return api.APIResult(result, api_result.timestamp, api_result.expires)
 
-    def messages(self):
+    @auto_call('char/MailMessages')
+    def messages(self, api_result=None):
         """Returns a list of headers for a character's mail."""
-        api_result = self.api.get('char/MailMessages',
-            {'characterID': self.char_id})
-
         rowset = api_result.result.find('rowset')
         results = []
         for row in rowset.findall('row'):
@@ -391,15 +363,14 @@ class Char(object):
 
         return api.APIResult(results, api_result.timestamp, api_result.expires)
 
-    def message_bodies(self, message_ids):
+    @auto_call('char/MailBodies', map_params={'message_ids': 'ids'})
+    def message_bodies(self, message_ids, api_result=None):
         """Returns the actual body content of a set of mail messages.
 
         NOTE: You *must* have recently looked up the headers of
         any messages you are requesting bodies for (via the 'messages'
         method) or else this call will fail.
         """
-        api_result = self.api.get('char/MailBodies',
-            {'characterID': self.char_id, 'ids': message_ids})
 
         rowset = api_result.result.find('rowset')
         results = {}
@@ -415,10 +386,9 @@ class Char(object):
 
         return api.APIResult(results, api_result.timestamp, api_result.expires)
 
-    def mailing_lists(self):
+    @auto_call('char/MailingLists')
+    def mailing_lists(self, api_result=None):
         """Returns the mailing lists to which a character is subscribed."""
-        api_result = self.api.get('char/MailingLists')
-
         rowset = api_result.result.find('rowset')
         results = {}
         for row in rowset.findall('row'):
@@ -427,11 +397,9 @@ class Char(object):
 
         return api.APIResult(results, api_result.timestamp, api_result.expires)
 
-    def calendar_events(self):
+    @auto_call('char/UpcomingCalendarEvents')
+    def calendar_events(self, api_result=None):
         """Returns the list of upcoming calendar events for a character."""
-        api_result = self.api.get('char/UpcomingCalendarEvents',
-            {'characterID': self.char_id})
-
         results = {}
         rowset = api_result.result.find('rowset')
         for row in rowset.findall('row'):
@@ -453,7 +421,8 @@ class Char(object):
 
         return api.APIResult(results, api_result.timestamp, api_result.expires)
 
-    def calendar_attendees(self, event_ids):
+    @auto_call('char/CalendarEventAttendees', map_params={'event_ids': 'eventIDs'})
+    def calendar_attendees(self, event_ids, api_result=None):
         """Returns the list of attendees for the specified calendar event.
 
         This function takes a list of event IDs and returns a dict of dicts,
@@ -463,8 +432,6 @@ class Char(object):
         NOTE: You must have recently fetched the list of calendar events
         (using the 'calendar_events' method) before calling this method.
         """
-        api_result = self.api.get('char/CalendarEventAttendees',
-            {'characterID': self.char_id, 'eventIDs': event_ids})
 
         results = dict((int(i),{}) for i in event_ids)
         rowset = api_result.result.find('rowset')
@@ -479,7 +446,7 @@ class Char(object):
 
         return api.APIResult(results, api_result.timestamp, api_result.expires)
 
-    def event_attendees(self, event_id):
+    def event_attendees(self, event_id, api_result=None):
         """Returns the attendees for a single event.
 
         (This is a convenience wrapper around 'calendar_attendees'.)
@@ -487,18 +454,18 @@ class Char(object):
         NOTE: You must have recently fetched the list of calendar events
         (using the 'calendar_events' method) before calling this method.
         """
+
         api_result = self.calendar_attendees([event_id])
         return api.APIResult(api_result.result[int(event_id)], api_result.timestamp, api_result.expires)
 
-    def faction_warfare_stats(self):
+    @auto_call('char/FacWarStats')
+    def faction_warfare_stats(self, api_result=None):
         """Returns FW stats for this character, if enrolled in FW.
 
         NOTE: This will return an error instead if the character
         is not enrolled in Faction Warfare.
-        """
-        api_result = self.api.get('char/FacWarStats',
-            {'characterID': self.char_id})
 
+        """
         _str, _int, _float, _bool, _ts = api.elem_getters(api_result.result)
 
         result = {
@@ -525,12 +492,9 @@ class Char(object):
 
         return api.APIResult(result, api_result.timestamp, api_result.expires)
 
-    def medals(self):
+    @auto_call('char/Medals')
+    def medals(self, api_result=None):
         """Returns a list of medals the character has."""
-
-        api_result = self.api.get('char/Medals',
-            {'characterID': self.char_id})
-
         result = {'current': {}, 'other': {}}
         _map = {
             'currentCorporation': 'current',
@@ -554,10 +518,9 @@ class Char(object):
 
         return api.APIResult(result, api_result.timestamp, api_result.expires)
 
-    def contact_notifications(self):
+    @auto_call('char/ContactNotifications')
+    def contact_notifications(self, api_result=None):
         """Returns pending contact notifications."""
-        api_result = self.api.get('char/ContactNotifications', {'characterID': self.char_id})
-
         results = {}
         rowset = api_result.result.find('rowset')
         for row in rowset.findall('row'):
@@ -575,12 +538,8 @@ class Char(object):
 
         return api.APIResult(results, api_result.timestamp, api_result.expires)
 
-    def locations(self, location_list):
-        params={
-            'IDs' : location_list,
-            'characterID' : self.char_id,
-        }
-        api_result = self.api.get('char/Locations', params)
+    @auto_call('char/Locations', map_params={'location_list': 'IDs'})
+    def locations(self, location_list, api_result=None):
         rowset = api_result.result.find('rowset')
         rows = rowset.findall('row')
 
